try:
  import gmsh
except (ImportError,OSError) as e:
  raise e

import cadquery as cq
import os
import tempfile
import math
from .assemblymesher_base import assemblymesher

class MesherGMSH(assemblymesher):
  gmsh_mesher_entities=None


  def __init__(self, min_mesh_size, max_mesh_size, curve_samples, default, mesh_algorithm, vetoed, threads, radial_threshold, refine, entities):
    self.IntermediateLayer='brep'
    self.min_mesh_size=min_mesh_size
    self.max_mesh_size=max_mesh_size
    self.curve_samples=curve_samples
    self.mesh_algorithm=mesh_algorithm
    self.default=default
    self._set_entities(entities)
    self.vetoed=vetoed
    self.threads=threads
    self.radial_threshold=radial_threshold
    self.refine=refine
    self._gmsh_init()
    self._cq_solids_to_gmsh()

  @classmethod
  def _set_entities(cls,entities):
    cls.gmsh_mesher_entities=entities

  @property
  def refine(self):
    return self._refine

  @refine.setter
  def refine(self,ref):
    if(ref==True):
      self._refine=1
    elif(ref==False):
      self._refine=0
    else:
      #assume ref is integer
      self._refine=abs(int(ref))

  def __del__(self):
    gmsh.finalize()

  def _gmsh_init(self):
      if not gmsh.isInitialized():
        gmsh.initialize()

      if (self.verbosity_level>1):
          gmsh.option.setNumber("General.Terminal",1)
      else:
          gmsh.option.setNumber("General.Terminal",0)

      if(not self.default):
        gmsh.option.setString("Geometry.OCCTargetUnit","CM")
        #do this by means of properties instead
        if(self.threads is not None):
          gmsh.option.setNumber("General.NumThreads",self.threads)

        gmsh.option.setNumber("Mesh.Algorithm", self.mesh_algorithm)
        gmsh.option.setNumber("Mesh.MeshSizeMin", self.min_mesh_size)
        gmsh.option.setNumber("Mesh.MeshSizeMax", self.max_mesh_size)

        gmsh.option.setNumber("Mesh.MaxRetries",3)
        gmsh.option.setNumber("Mesh.MeshSizeFromPoints",1)
        gmsh.option.setNumber("Mesh.MeshSizeExtendFromBoundary", 1)
        gmsh.option.setNumber("Mesh.MeshSizeFromCurvature", self.curve_samples)
      if self.radial_threshold>0:
        self._set_radial_field()

  def _set_pars(self,min_mesh_size, max_mesh_size, curve_samples, default, mesh_algorithm, vetoed, threads, radial_threshold, refine):
    self.min_mesh_size=min_mesh_size
    self.max_mesh_size=max_mesh_size
    self.curve_samples=curve_samples
    self.mesh_algorithm=mesh_algorithm
    self.default=default
    self.vetoed=vetoed
    self.threads=threads
    self.radial_threshold=radial_threshold
    self.refine=refine
    self._gmsh_init()

  def _set_radial_field(self):
    #function to set up a radial field for meshing making the
    #mesh size depend inversely on the distance from the z-axis
    gmsh.model.mesh.setSizeCallback(_radial_field)

  def _cq_solids_to_gmsh(self):
      import glob
      #create a compund cq solid from entities
      solids=[e.solid for e in self.gmsh_mesher_entities]
      compound=cq.Compound.makeCompound(solids)
      with tempfile.TemporaryDirectory() as td:
        outpath=os.path.join(td,'export.'+self.IntermediateLayer)
        compound.exportBrep(outpath)
        vols=gmsh.model.occ.importShapes(outpath)
      #vols = gmsh.model.occ.importShapesNativePointer(compound)
      gmsh.model.occ.synchronize()
      self._reorder()

  def _reorder(self):
      #in the process of exporting/importing, the ordering may have been jumbled
      brep_volume_dimtags=gmsh.model.occ.getEntities(3)
      tag_idx=[]
      for dimtag in brep_volume_dimtags:
        cms=gmsh.model.occ.get_center_of_mass(dimtag[0],dimtag[1])
        bb0=gmsh.model.occ.get_bounding_box(dimtag[0],dimtag[1])
        bb=[abs(bb0[0]-bb0[3]),abs(bb0[1]-bb0[4]),abs(bb0[2]-bb0[5])]
        vol=gmsh.model.occ.get_mass(dimtag[0],dimtag[1])
        j=self._find_similar(cms,bb,vol)
        tag_idx.append(j)
      #tag_idx is now the reordered list of indices
      #use that to reorder the entities to match the newly imported
      #list of volumes in the gmsh geometry representation
      self.gmsh_mesher_entities=[self.gmsh_mesher_entities[i] for i in tag_idx if i!=-1]

  def _find_similar(self,cms,bb,vol):
      closest=1e12
      iclose=-1
      for i,e in enumerate(self.gmsh_mesher_entities):
          simi=e.similarity(cms,bb,vol,tolerance=1e5)
          if (simi<closest):
              iclose=i
              closest=simi
      return iclose

  def _generate_mesh(self):
      if(self.verbosity_level>0):
          print("INFO: GMSH generate surface mesh")
      gmsh.model.mesh.generate(2)
      for i in range(self.refine):
        gmsh.model.mesh.refine()

  def generate_stls(self):
      """export all the optionally merged volumes as stl-files
      and returns the list of files. This list may subsequently be iterated upon
      to merge into a h5m-file. Expects that the geometry has been surface-mesh by gmsh
      so we have a list of volumes to operate on.
      We do this be greating gmsh physical groups and export 1 group at a time."""
      self._generate_mesh()
      stls=[]
      vols=gmsh.model.getEntities(3)
      for k,e in enumerate(self.gmsh_mesher_entities):
        #gmsh volume ids run from 1
        dim,vid=vols[k]

        if (dim!=3):
            #appears not to be a volume - skip
            continue
        ents = gmsh.model.getAdjacencies(dim,vid)
        #may want to skip some problematic surfaces
        if(self.vetoed):
            picked_ents=[f for f in ents[1] if f not in self.vetoed]
        else:
            picked_ents=ents[1]
        pg = gmsh.model.addPhysicalGroup(2,picked_ents)
        ps = gmsh.model.setPhysicalName(2,pg,f'surfaces_on_volume_{vid}')
        filename=f'volume_{vid}.stl'
        try:
            gmsh.write(filename)
            stls.append(filename)
        except:
            stls.append('None')
            print(f'WARNING: Could not write volume {vid}. Skipping')
        gmsh.model.removePhysicalGroups([]) # remove group again
      return stls

class MesherGMSHBuilder:
  def __init__(self):
    self._instance = None

  def __call__(self, min_mesh_size, max_mesh_size, curve_samples, default, mesh_algorithm, vetoed, threads, radial_threshold, refine, entities, **_ignored):
    if not self._instance:
      self._instance = MesherGMSH(min_mesh_size, max_mesh_size, curve_samples, default, mesh_algorithm, vetoed, threads, radial_threshold, refine, entities)
    else:
      #need to do it this way since gmsh needs to be reinitialized
      self._instance._set_entities(entities)
      self._instance._set_pars(min_mesh_size,max_mesh_size, curve_samples, default, mesh_algorithm, vetoed, threads, radial_threshold, refine)
    return self._instance

def _radial_field(dim,tag, x, y, z, lc):
  if(math.sqrt(x*x+y*y)>lc):
    return lc/math.sqrt(x*x+y*y)
  else:
    return lc



import pathlib
import os
import openmc
import math

import CAD_to_OpenMC.assembly as ab

from tests.OMC_DAGMC_run import DAGMC_template
from tests.harnessRun import HarnessRun

class OMC_DAGMC_harness(HarnessRun):
  def __init__(self, step):
    self.path = pathlib.Path(step)
    self.h5m = self.path.with_suffix('.h5m')
    self.nuclear_lib = pathlib.Path('tests/nuclear_data_testlib/cross_sections.xml').absolute()
    self.aa = ab.Assembly([str(self.path)], verbose = 2)
    self.tt = DAGMC_template(self.h5m)

  def run(self):
    self.aa.run(backend='stl2', merge = True, h5m_filename = self.h5m)
    assert self.h5m.exists()
    self.tt.run()
    assert pathlib.Path('statepoint.5.h5').exists()
    self.tt.cleanup()
    self.cleanup()

def test_h5m_neutronics_p1():
  o = OMC_DAGMC_harness('examples/pincell1.step')
  openmc.config['cross_sections']=str(o.nuclear_lib)
  o.run()

def test_h5m_neutronics_p2():
  o = OMC_DAGMC_harness('examples/pincell2.step')
  openmc.config['cross_sections']=str(o.nuclear_lib)
  o.run()

def test_h5m_neutronics_tors():
  o = OMC_DAGMC_harness('examples/toroids.step')
  # override source spatial distribution
  o.tt.settings.source.space=openmc.stats.CylindricalIndependent(
    openmc.stats.Discrete([100,85,77.5],[1.0/3.0,1.0/3.0, 1.0/3.0]), openmc.stats.Uniform(0.0,2.0*math.pi), openmc.stats.Discrete([0.0],[1.0])
  )
  openmc.config['cross_sections']=str(o.nuclear_lib)
  o.run()

def test_h5m_neutronics_spheroids():
  o = OMC_DAGMC_harness('examples/spheroids.step')
  openmc.config['cross_sections']=str(o.nuclear_lib)
  o.run()

def test_h5m_neutronics_ellipsoids():
  o = OMC_DAGMC_harness('examples/oblate_ellipsoids.step')
  openmc.config['cross_sections']=str(o.nuclear_lib)
  o.run()

if __name__=='__main__':
  test_h5m_neutronics_p1()
  test_h5m_neutronics_p2()
  test_h5m_neutronics_tors()
  test_h5m_neutronics_spheroids()
  test_h5m_neutronics_ellipsoids()



import cadquery as cq
import subprocess as sp
import pathlib as pl
from .assemblymesher_base import assemblymesher

single_thread_override=False
try:
  import multiprocessing as mp
  manager = mp.Manager()
  lock = manager.Lock()
except:
  single_thread_override=True

from .stl_utils import *
from . import meshutils

class MesherCQSTL2(assemblymesher):
  #these need to be class attributes to avoid pickling when spawning a multiprocessing-pool
  cq_mesher_entities=None
  cq_mesher_tolerance=None
  cq_mesher_ang_tolereance=None
  cq_mesher_min_mesh_size=None
  cq_mesher_max_mesh_size=None

  cq_mesher_faceHash={}

  def __init__(self, tolerance, angular_tolerance, min_mesh_size, max_mesh_size, default, refine, threads, entities):
    self._set_meshpars(tolerance,angular_tolerance,min_mesh_size,max_mesh_size)
    self._clear_face_hashtable()
    self.refine=refine
    self._set_entities(entities)
    self.default=default
    self.min_mesh_size=min_mesh_size
    self.max_mesh_size=max_mesh_size
    if(len(entities)<=threads):
      self.threads=len(entities)
    else:
      self.threads=threads

  @property
  def refine(self):
    return self._refine

  @refine.setter
  def refine(self,ref):
    if(ref==True or ref!=0):
      self._refine=True
    else:
      self._refine=False

  @classmethod
  def _set_meshpars(cls,tol,ang_tol, min_sz, max_sz):
    cls.cq_mesher_tolerance=tol
    cls.cq_mesher_ang_tolerance=ang_tol
    cls.cq_mesher_min_mesh_size=min_sz
    cls.cq_mesher_max_mesh_size=max_sz

  @classmethod
  def _clear_face_hashtable(cls):
    cls.cq_mesher_faceHash={}

  @classmethod
  def _set_entities(cls,entities):
    cls.cq_mesher_entities=entities

  def generate_stls(self):
    return self._mesh_surfaces()

  def _mesh_surfaces(self):
    #loop over all surfaces in all entities
    #and generate meshes (refined or otherwise)
    mpargs=[]
    if (single_thread_override or self.threads==1):
      face_hash_table={}
    else:
      #manager=mp.Manager()
      face_hash_table=manager.dict()
    k=0
    for i,e in enumerate(self.cq_mesher_entities):
      mpargs.extend([(k+j,j,i,self.refine,hash(f), face_hash_table) for j,f in enumerate(e.solid.Faces())])

    #we have a set of mesh jobs - scatter those
    if (single_thread_override or self.threads==1):
      output=[]
      for args in mpargs:
        output.append(self._mesh_single_nothread(*args))
    else:
      pool=mp.Pool(processes=self.threads)
      output=pool.starmap(self._mesh_single, mpargs)

    #process the list of meshed faces.
    stls=[]
    for i,e in enumerate(self.cq_mesher_entities):
      face_stls=[]
      for k,v in face_hash_table.items():
        vids=v[1] # the volumes that this face belongs to
        if i in vids:
          #this face is part of this volume
          face_stls.append(v)
      stls.append(face_stls)
    return stls

  @classmethod
  def _mesh_single_nothread(cls, global_fid, fid, vid, refine, hh, faceHash):
    f=cls.cq_mesher_entities[vid].solid.Faces()[fid]
    if hh in faceHash.keys():
      #surface is in table - simply add the vid to the hash-table
      faceHash[hh][1].append(vid)
      if (cls.verbosity_level):
        print(f'INFO: mesher reusing {hh} {faceHash[hh][1]}')
      return(hh,faceHash[hh])
    else:
      facefilename=f'vol_{vid+1}_face{global_fid:04}.stl'
      faceHash[hh]=[facefilename,manager.list([vid])]
      f.exportStl(facefilename, tolerance=cls.cq_mesher_tolerance, angularTolerance=cls.cq_mesher_ang_tolerance, ascii=True)
      if(cls.verbosity_level>1):
        print(f"INFO: cq export to file {facefilename}")
      if (refine):
        cls._refine_stls(facefilename,refine)
      return(hh,faceHash[hh])

  @classmethod
  def _mesh_single(cls, global_fid, fid, vid, refine, hh, faceHash):
    f=cls.cq_mesher_entities[vid].solid.Faces()[fid]
    if hh in faceHash.keys():
      #surface is in table - simply add the vid to the hash-table
      with lock:
        faceHash[hh][1].append(vid)
      if (cls.verbosity_level):
        print(f'INFO: mesher reusing {hh} {faceHash[hh][1]}')
      return(hh,faceHash[hh])
    else:
      facefilename=f'vol_{vid+1}_face{global_fid:04}.stl'
      with lock:
        faceHash[hh]=[facefilename,manager.list([vid])]
      f.exportStl(facefilename, tolerance=cls.cq_mesher_tolerance, angularTolerance=cls.cq_mesher_ang_tolerance, ascii=True)
      if(cls.verbosity_level>1):
        print(f"INFO: cq export to file {facefilename}")
      if (refine):
        cls._refine_stls(facefilename,refine)
      return(hh,faceHash[hh])

class MesherCQSTL2Builder:
  def __init__(self):
    self._instance = None

  def __call__(self, tolerance, angular_tolerance, min_mesh_size, max_mesh_size, default, refine, threads, entities, **_ignored):
    if not self._instance:
      self._instance = MesherCQSTL2(tolerance, angular_tolerance, min_mesh_size, max_mesh_size, default,refine, threads, entities)
    else:
      #We are reusing a mesher instance. Hence reset the parameters and clear the hashtable
      self._instance._set_entities(entities)
      self._instance._set_meshpars(tolerance, angular_tolerance, min_mesh_size, max_mesh_size)
      self._instance._clear_face_hashtable()
    return self._instance



import pytest
import CAD_to_OpenMC.assembly as ab
from tests.harnessRun import HarnessRun
import pathlib as pl
import sys

class HarnessCqSTL2(HarnessRun):
    def __init__(self):
        super().__init__()

    def run(self,merge=False):
        if merge:
            self.merge()

        #self.a.solids_to_h5m(backend='stl')
        h5p = pl.Path('out_cqstl.h5m')
        self.a.solids_to_h5m(backend='stl2',h5m_filename=str(h5p))
        assert h5p.exists()
        assert self.is_validh5m(h5p)
        self.cleanup()

    def cleanup(self):
        super().cleanup()
        pwd=pl.Path('.')
        for v in pwd.glob("vol*_face*"):
            v.unlink()

def testcq():
    t = HarnessCqSTL2()
    t.run()

def testcq_wmerge():
    t = HarnessCqSTL2()
    t.run(merge=True)

if __name__=='__main__':
    testcq()
    testcq_wmerge()



import pytest
import CAD_to_OpenMC.assembly as ab
from tests.harnessRun import HarnessRun
import pathlib as pl

class HarnessGmsh(HarnessRun):
  def __init__(self):
    super().__init__()

  def run(self):
    print(f'gmsh: {self.a.stp_files})')
    self.a.merge_all()
    ab.mesher_config['refine']=False
    h5p = pl.Path('out_gmsh.h5m')
    self.a.solids_to_h5m(backend='gmsh',h5m_filename=str(h5p))
    assert h5p.exists()
    assert self.is_validh5m(h5p)
    #self.cleanup()

def testgmsh():
  t = HarnessGmsh()
  t.run()



class ObjectFactory:
    def __init__(self):
        self._builders = {}

    def register_builder(self, key, builder):
        self._builders[key] = builder

    def create(self, key, **kwargs):
        builder = self._builders.get(key)
        if not builder:
            raise ValueError(key)
        return builder(**kwargs)



import abc
from .assemblymesher_base import assemblymesher
from .object_factory import ObjectFactory

nogmsh=False
try:
  from .assemblymesher_gmsh import MesherGMSHBuilder
except (ImportError,OSError) as e:
  nogmsh=e

nocq=False
try:
  from .assemblymesher_cq import MesherCQSTLBuilder
except (ImportError,OSError) as e:
  nocq=e

nocq2=False
try:
  from .assemblymesher_cq2 import MesherCQSTL2Builder
except (ImportError,OSError) as e:
  npcq2=e

if (nogmsh and nocq and nocq2):
  raise ImportError("Could not import any of the mesher backends")

class MesherFactory(ObjectFactory):
  def get(self,mesher_id,**kwargs):
    return self.create(mesher_id, **kwargs)

meshers=MesherFactory()
if(not nogmsh):
  meshers.register_builder('gmsh',MesherGMSHBuilder())
if(not nocq):
  meshers.register_builder('stl',MesherCQSTLBuilder())
if(not nocq2):
  meshers.register_builder('stl2',MesherCQSTL2Builder())



import h5py
import numpy as np


def _get_vertices(h5f):
  verts_ds=h5f['tstt']['nodes']['coordinates']
  return verts_ds

def _modify_vertices(v,tol=1e-6):
  for i in range(len(v)):
    print(f'{i} of {len(v)}')
    for j in range(i+1,len(v)):
      delta=np.linalg.norm(v[i]-v[j])
      if (delta<tol and delta>0):
        print(f'close but no cigar {i}:{v[i]} {j}:{v[j]} {delta}')
  return v

def heal_h5m(filename:str='dagmc.h5m'):
  with h5py.File(filename,'r+') as h5f:
    verts=_get_vertices(h5f)
    healed=_modify_vertices(verts,tol=1e-6)

if __name__=='__main__':
  heal_h5m() 



import cadquery as cq
import subprocess as sp
import pathlib as pl
import hashlib as hl
from CAD_to_OpenMC import cdtemp
from .assemblymesher_base import assemblymesher

single_thread_override=True
try:
  import multiprocessing.pool as mp
except:
  single_thread_override=True

from .stl_utils import *
from .meshutils import *

#following pattern here: https://thelaziestprogrammer.com/python/multiprocessing-pool-a-global-solution
#to enable multiprocessing meshings

class MesherCQSTL(assemblymesher):
  #these need to be class attributes to avoid pickling when spawning a multiprocessing-pool
  cq_mesher_entities=None
  cq_mesher_tolerance=None
  cq_mesher_ang_tolereance=None
  cq_mesher_min_mesh_size=None
  cq_mesher_max_mesh_size=None

  cq_mesher_faceHash={}

  def __init__(self, tolerance, angular_tolerance, min_mesh_size, max_mesh_size, default, refine, threads, entities):
    self._set_meshpars(tolerance,angular_tolerance,min_mesh_size,max_mesh_size)
    self._clear_face_hashtable()
    self.refine=refine
    self._set_entities(entities)
    self.default=default
    self.min_mesh_size=min_mesh_size
    self.max_mesh_size=max_mesh_size
    self.threads=threads

  @property
  def refine(self):
    return self._refine

  @refine.setter
  def refine(self,ref):
    if(ref==True or ref!=0):
      self._refine=True
    else:
      self._refine=False

  @classmethod
  def _set_meshpars(cls,tol,ang_tol, min_sz, max_sz):
    cls.cq_mesher_tolerance=tol
    cls.cq_mesher_ang_tolerance=ang_tol
    cls.cq_mesher_min_mesh_size=min_sz
    cls.cq_mesher_max_mesh_size=max_sz

  @classmethod
  def _clear_face_hashtable(cls):
    cls.cq_mesher_faceHash={}

  @classmethod
  def _set_entities(cls,entities):
    cls.cq_mesher_entities=entities

  def generate_stls(self):
    stls=self._mesh_surfaces()
    return stls

  @classmethod
  def _refine_stls(cls,stl,refine):
    try:
      if(refine.startswith('mmg')):
        cls._mmgs_refine_stl(stl)
      else:
        print(f'ERROR: Unknown mesh refinement algorithm: {self.refine}')
    except:
        cls._mmgs_refine_stl(stl)

  @classmethod
  def _mmgs_refine_stl(cls,stl):
    #call mmgs to refine the mesh
    try:
      cp=sp.run(['mmgs_O3','--help'],capture_output=True)
    except:
      print(f'WARNING: Cannot find mmgs mesh refinement tool. Vol. {stl} will not be remeshed. Did you forget to include it on the PATH?')
      return
    stlp=pl.Path(stl)
    with open(stl) as fp:
      buf,n=read_stl_ascii(fp)
    verts=buffer2vertices(buf)
    tris=buffer2triangles(buf,verts)
    edges=np.array(find_edges(tris))
    write_dotmesh(str(stlp.with_suffix('.mesh')),verts,tris,edges=edges,required_edges='all')
    cp=sp.run(['mmgs_O3','-hmin',f'{cls.cq_mesher_min_mesh_size}','-hmax',f'{cls.cq_mesher_max_mesh_size}','-optim','-in',stlp.with_suffix('.mesh'),'-out',stlp.with_suffix('.o.mesh')], capture_output=True)
    if(cls.verbosity_level and cls.verbosity_level>1):
      print(cp.stdout.decode())

    import gmsh
    gmsh.initialize()
    gmsh.open(str(stlp.with_suffix('.o.mesh')))
    gmsh.write( str(stlp) )
    gmsh.finalize()

  def _mesh_surfaces(self):
    #loop over all surfaces in all entities
    #and generate meshes (refined or otherwise)
    stls=[]
    cwd=pl.Path.cwd()
    #create a workplace in tmp
    with cdtemp() as mngr:
      for i,e in enumerate(self.cq_mesher_entities):
        volname= f"volume_{i+1}.stl"
        mpargs=[(j,i,self.refine) for j,f in enumerate(e.solid.Faces())]
        if (single_thread_override):
          output=[]
          for args in mpargs:
            output.append(self._mesh_single(*args))
        else:
          pool=mp.Pool(processes=self.threads)
          output=pool.starmap(self._mesh_single, mpargs)
        volumefaces=[]
        for o in output:
          self.cq_mesher_faceHash[o[0]]=o[1]
          volumefaces.append(o[1])
        #merge the stls to a single .stl in the working directory
        merge_stl(str(cwd / volname), volumefaces,of='bin')
        stls.append(volname)
    # clear the hash table
    self._clear_face_hashtable()
    return stls

  @classmethod
  def _mesh_single(cls, fid, vid, refine):
    f=cls.cq_mesher_entities[vid].solid.Faces()[fid]
    hh=hash(f)
    if hh in cls.cq_mesher_faceHash.keys():
      #surface is in table - use that file for this volume
      if (cls.verbosity_level):
        print(f'INFO: mesher reusing {hh} {cls.cq_mesher_faceHash[hh]}')
      return(hh,cls.cq_mesher_faceHash[hh])
    else:
      facename=f'vol_{vid+1}_face{fid}.stl'
      cls.cq_mesher_faceHash[hh]=facename
      f.exportStl(facename, tolerance=cls.cq_mesher_tolerance, angularTolerance=cls.cq_mesher_ang_tolerance, ascii=True)
      if(cls.verbosity_level>1):
        print(f"INFO: cq export to file {facename}")
      if (refine):
        cls._refine_stls(facename,refine)
      return(hh,facename)

class MesherCQSTLBuilder:
  def __init__(self):
    self._instance = None

  def __call__(self, tolerance, angular_tolerance, min_mesh_size, max_mesh_size, default, refine, threads, entities, **_ignored):
    if not self._instance:
      self._instance = MesherCQSTL(tolerance, angular_tolerance, min_mesh_size, max_mesh_size, default,refine, threads, entities)
    else:
      #We are reusing a mesher instance. Hence reset the parameters and clear the hashtable
      self._instance._set_entities(entities)
      self._instance._set_meshpars(tolerance, angular_tolerance, min_mesh_size, max_mesh_size)
      self._instance._clear_face_hashtable()
    return self._instance



import pytest
import CAD_to_OpenMC.assembly as ab
from tests.harnessRun import HarnessRun
import pathlib as pl

class HarnessTrans(HarnessRun):
  def __init__(self):
    self.infile = 'examples/7pin.step'
    self.a = ab.Assembly(verbose=2)

  def run(self,merge=False, **kwargs):
    self.a.stp_files=[self.infile]
    print(f'stl: {self.a.stp_files})')
    self.a.import_stp_files(**kwargs)

    if merge:
      self.merge()

    #self.a.solids_to_h5m(backend='stl')
    h5p = pl.Path('out_transformed.h5m')
    self.h5m=str(h5p)

    self.a.solids_to_h5m(backend='stl',h5m_filename=str(h5p))
    assert h5p.exists()
    assert self.is_validh5m(h5p)
    self.cleanup()

  def cleanup(self):
    super().cleanup()
    pwd=pl.Path('.')
    for v in pwd.glob("vol*_face*"):
      v.unlink()

def test_scale():
  t = HarnessTrans()
  t.run(scale = 2.0)

def test_translate():
  t = HarnessTrans()
  t.run(translate = [[1,2],[0.0, 0.0, -30.0],[3],[0.0, 0.0, 30]])



import pytest
import CAD_to_OpenMC.assembly as ab
from tests.harnessRun import HarnessRun
import pathlib as pl

class HarnessCqSTL(HarnessRun):
  def __init__(self):
    super().__init__()

  def run(self,merge=False):
    print(f'stl: {self.a.stp_files})')
    print(self.h5m)

    if merge:
      self.merge()

    #self.a.solids_to_h5m(backend='stl')
    h5p = pl.Path('out_cqstl.h5m')
    self.a.solids_to_h5m(backend='stl',h5m_filename=str(h5p))
    assert h5p.exists()
    assert self.is_validh5m(h5p)
    self.cleanup()

  def cleanup(self):
    super().cleanup()
    pwd=pl.Path('.')
    for v in pwd.glob("vol*_face*"):
      v.unlink()

def testcq():
  t = HarnessCqSTL()
  t.run()

def testcq_wmerge():
  t = HarnessCqSTL()
  t.run(merge=True)
